/* CUnitTester http://www.gethos.net/opensource/cunit
 * show_colour.c -- handles all normal colour output
 *
 * Copyright 2003 by Stewart Gebbie
 * For license terms, see the file COPYING in the top directory.
 */
/* $Id: show_colour.c,v 1.6 2005/01/05 20:53:56 stewart Exp $ */
#include "config.h"
#include "unittester.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "show_colour.h"

#define COLOUR_NONE 0
#define COLOUR_RED 1
#define COLOUR_GREEN 2
#define COLOUR_YELLOW 3
#define COLOUR_BLUE 4
#define COLOUR_NORMAL 5

static const char* coloursDefault[] = {
	"",
	"\033[01;31m",
	"\033[01;32m",
	"\033[01;33m",
	"\033[01;34m",
	"\033[00m"
};

static const char* coloursMono[] = {
	"",
	"",
	"",
	"",
	"",
	""
};

static const char** colours = NULL;

void colour_ShowInit ()
{
	const char* termEnv = getenv ("TERM");
	colours = coloursMono;
	if (
			(termEnv != NULL)
			&& (
				(strcmp (termEnv, "xterm") == 0)
				|| (strcmp (termEnv, "kterm") == 0)
				|| (strcmp (termEnv, "linux") == 0)
				|| (strcmp (termEnv, "rxvt") == 0)
			)
		)
	{
		colours = coloursDefault;
	}
}

void colour_ShowFini ()
{
	colours = NULL;
}

void colour_ShowError_va (const char* fmt, va_list ap)
{
	printf ("%s", colours[COLOUR_RED]);
	vprintf (fmt, ap);
	printf ("%s", colours[COLOUR_NORMAL]);
}

void colour_ShowMsg_va (const char* fmt, va_list ap)
{
	printf ("%s", colours[COLOUR_YELLOW]);
	vprintf (fmt, ap);
	printf ("%s", colours[COLOUR_NORMAL]);
}

void colour_ShowUsage (int argc, char* argv[], const char* errmsg)
{
	printf ("Usage: %s [-h] [-r search directory] [-f filter regex]\n", argv[0]);
	if (errmsg) printf ("Error: %s\n", errmsg);
	printf ("\n"
			"%s\n"
			"\tURL: %s\n"
			"\n"
			"A framework for running unit tests written in C\n",
			CUNIT_BANNER,
			CUNIT_URL
			);
	printf (
			"\n"
			"\tStarting in the specified search directory, CUnitTester\n"
			"\twill recurse through the sub directories and search for\n"
			"\tfiles with names matching libtest_*.so.\n"
			"\tThe test suites defined within these libraries and matching\n"
			"\tthe filter expression are then loaded and run.\n"
			"\tThe final statistics are displayed along with any\n"
			"\taccumulated error messages.\n"
			);
}


static void colour_ShowTestInfo (utest_info* info)
{
	if (info->status == false)
		printf("\t%s - %s %s:%d - %s\n", 
				info->suiteName, 
				info->testName, 
				info->file, 
				info->line, 
				info->msg);
}


void colour_ShowInfo (utest_world* world)
{
	double percentGood;
	double percentBad;
	double runTime;

	utest_info* current;

	if (world->badTestCount > 0)
		printf ("\n%sErrors%s:\n", colours[COLOUR_YELLOW], colours[COLOUR_NORMAL]);

	for (current = world->infoHead; current != NULL; current = current->next)
		colour_ShowTestInfo (current);

	percentGood = 100.0*((double)world->goodTestCount/(double)world->testCount);
	percentBad = 100.0*((double)world->badTestCount/(double)world->testCount);

	printf ("\n%sSummary%s:\n", colours[COLOUR_YELLOW], colours[COLOUR_NORMAL]);
	printf ("\tTotal Tests %d\n", world->testCount);
	printf ("\tGood Tests %d %3.2f%% \n", world->goodTestCount, percentGood);
	printf ("\tBad Tests %d %3.2f%% \n", world->badTestCount, percentBad);

	/* calculate times */
	runTime = (world->secEnd + ((double)world->usecEnd/(double)1000000))
		- (world->secStart + ((double)world->usecStart/(double)1000000));
	printf ("\tRun Time %3.3f secs\n", runTime);

	printf ("\n%sBottom Line%s:\n", colours[COLOUR_YELLOW], colours[COLOUR_NORMAL]);
	printf ("\t%s[%d/%d]%s tests passed.\n\t%s%3.2f%%%s.\n", 
			colours[COLOUR_BLUE],
			world->goodTestCount, 
			world->testCount, 
			colours[COLOUR_NORMAL],
			(world->goodTestCount == world->testCount?colours[COLOUR_GREEN]:colours[COLOUR_RED]),
			percentGood,
			colours[COLOUR_NORMAL]
			);
}


void colour_ShowStart (utest_world* world)
{
	printf ("There %s %d test%s in %d suite%s to run:\n", 
			world->testCount == 1 ? "is" : "are", 
			world->testCount, 
			world->testCount == 1 ? "" : "s", 
			world->suiteCount,
			world->suiteCount == 1 ? "" : "s"
			);
}

static char swizzle[]={
	'-','\\','|','/',
	'\0'
};

void colour_ShowProgress (utest_world* world, utest_test* test)
{
	int perc = 0;
	int width = 0;
	int good = 0;
	int bad = 0;
	int blank = 0;
	const char* theColour = NULL;
	int i;

	if (world->testCount == 0)
	{
		return;
	}

	/* draw swizzle */
	printf("%c ", swizzle[world->finTestCount%(sizeof(swizzle)-1)]);

	/* draw % */
	printf("[%d/%d] ",world->finTestCount,world->testCount);

	/* draw % */
	perc=(int)(100*(double)world->finTestCount/(double)world->testCount);
	printf("%3d%% ",perc);

	/* draw progress bar */
	width=55;
	good=(int)ceil(width*((double)world->goodTestCount/
				(double)world->testCount));
	bad=(int)ceil(width*((double)world->badTestCount/
				(double)world->testCount));
	blank=width - (good + bad);/* - (good?1:0) - (bad?1:0); */

	theColour = (bad>0 ? colours[COLOUR_RED] : colours[COLOUR_GREEN]);

	printf("<%s", theColour);
	for(i=0;i<bad;i++) printf("-");
	/* for(int i=0;i<err;i++) cout<<":"; */
	for(i=0;i<good;i++) printf("=");
	if (perc != 100)
	{
		printf (">");
		blank--;
	}
	for(i=0;i<blank;i++) printf(" ");
	printf("%s>\r", colours[COLOUR_NORMAL]);

#if 0
	printf("\n");

	printf ("t:%d f:%d g:%d b:%d\n",world->testCount, 
			world->finTestCount,
			world->goodTestCount, 
			world->badTestCount);
#endif

	/* end progress bar */
	fflush (stdout);
}


void colour_ShowBanner (utest_world* world)
{
	printf ("%s%s%s:\n", colours[COLOUR_YELLOW], CUNIT_BANNER, colours[COLOUR_NORMAL]);
	printf ("\t" CUNIT_URL "\n");
	printf ("Search Directory [%s]\n", world->rootDir);
	if (world->filter)
	{
		printf ("Search Filter [%s]\n", world->filter);
	}
}


#define MAX_SHOW_PATH_LEN 100
#define MAX_SHOW_NAME_LEN 100
void colour_ShowTestMessage (utest_suite_stub* stub, const char* msg)
{
	char path[MAX_SHOW_PATH_LEN];
	char name[MAX_SHOW_NAME_LEN];

	snprintf (path, sizeof(path), "[%-30.30s]", stub->libpath);
	snprintf (name, sizeof(path), "%s", stub->suite->name);
	printf ("%s %s %-20.20s %2d test%s\n", 
			msg,
			path,
			name,
			stub->testCount,
			stub->testCount == 1 ? "" : "s");
}


void colour_ShowAddTest (utest_suite_stub* stub)
{
	colour_ShowTestMessage (stub, "Adding suite  ");
}



void colour_ShowSkipTest (utest_suite_stub* stub)
{
	colour_ShowTestMessage (stub, "Skipping suite");
}



static ShowFunctions colourShowFunctions =
{
	&colour_ShowInit,
	&colour_ShowFini,
	&colour_ShowError_va,
	&colour_ShowMsg_va,
	&colour_ShowUsage,
	&colour_ShowBanner,
	&colour_ShowAddTest,
	&colour_ShowSkipTest,
	&colour_ShowInfo,

	NULL,
	NULL,
	&colour_ShowStart,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&colour_ShowProgress,
	&colour_ShowProgress,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

ShowFunctions* getColourShowFunctions()
{
	return &colourShowFunctions;
}
