/* CUnitTester http://www.gethos.net/opensource/cunit
 * show_mono.c -- handles all normal mono output
 *
 * Copyright 2003 by Stewart Gebbie
 * For license terms, see the file COPYING in the top directory.
 */
/* $Id: show_mono.c,v 1.1 2005/01/06 16:57:01 stewart Exp $ */
#include "config.h"
#include "unittester.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "show_mono.h"


void mono_ShowInit ()
{
}

void mono_ShowFini ()
{
}

void mono_ShowError_va (const char* fmt, va_list ap)
{
	vprintf (fmt, ap);
}

void mono_ShowMsg_va (const char* fmt, va_list ap)
{
	vprintf (fmt, ap);
}

void mono_ShowUsage (int argc, char* argv[], const char* errmsg)
{
	printf ("Usage: %s [-h] [-r search directory] [-f filter regex]\n", argv[0]);
	if (errmsg) printf ("Error: %s\n", errmsg);
	printf ("\n"
			"%s\n"
			"\tURL: %s\n"
			"\n"
			"A framework for running unit tests written in C\n",
			CUNIT_BANNER,
			CUNIT_URL
			);
	printf (
			"\n"
			"\tStarting in the specified search directory, CUnitTester\n"
			"\twill recurse through the sub directories and search for\n"
			"\tfiles with names matching libtest_*.so.\n"
			"\tThe test suites defined within these libraries and matching\n"
			"\tthe filter expression are then loaded and run.\n"
			"\tThe final statistics are displayed along with any\n"
			"\taccumulated error messages.\n"
			);
}


static void mono_ShowTestInfo (utest_info* info)
{
	if (info->status == false)
		printf("\t%s - %s %s:%d - %s\n", 
				info->suiteName, 
				info->testName, 
				info->file, 
				info->line, 
				info->msg);
}


void mono_ShowInfo (utest_world* world)
{
	double percentGood;
	double percentBad;
	double runTime;

	utest_info* current;

	if (world->badTestCount > 0)
		printf ("\nErrors:\n");

	for (current = world->infoHead; current != NULL; current = current->next)
		mono_ShowTestInfo (current);

	percentGood = 100.0*((double)world->goodTestCount/(double)world->testCount);
	percentBad = 100.0*((double)world->badTestCount/(double)world->testCount);

	printf ("\nSummary:\n");
	printf ("\tTotal Tests %d\n", world->testCount);
	printf ("\tGood Tests %d %3.2f%% \n", world->goodTestCount, percentGood);
	printf ("\tBad Tests %d %3.2f%% \n", world->badTestCount, percentBad);

	/* calculate times */
	runTime = (world->secEnd + ((double)world->usecEnd/(double)1000000))
		- (world->secStart + ((double)world->usecStart/(double)1000000));
	printf ("\tRun Time %3.3f secs\n", runTime);

	printf ("\nBottom Line:\n");
	printf ("\t[%d/%d] tests passed.\n\t%3.2f%%.\n", 
			world->goodTestCount, 
			world->testCount,
			percentGood
			);
}


void mono_ShowStart (utest_world* world)
{
	printf ("There %s %d test%s in %d suite%s to run:\n", 
			world->testCount == 1 ? "is" : "are", 
			world->testCount, 
			world->testCount == 1 ? "" : "s", 
			world->suiteCount,
			world->suiteCount == 1 ? "" : "s"
			);
}

static char swizzle[]={
	'-','\\','|','/',
	'\0'
};

void mono_ShowProgress (utest_world* world, utest_test* test)
{
	int perc = 0;
	int width = 0;
	int good = 0;
	int bad = 0;
	int blank = 0;
	int i;

	if (world->testCount == 0)
	{
		return;
	}

	/* draw swizzle */
	printf("%c ", swizzle[world->finTestCount%(sizeof(swizzle)-1)]);

	/* draw % */
	printf("[%d/%d] ",world->finTestCount,world->testCount);

	/* draw % */
	perc=(int)(100*(double)world->finTestCount/(double)world->testCount);
	printf("%3d%% ",perc);

	/* draw progress bar */
	width=55;
	good=(int)ceil(width*((double)world->goodTestCount/
				(double)world->testCount));
	bad=(int)ceil(width*((double)world->badTestCount/
				(double)world->testCount));
	blank=width - (good + bad);/* - (good?1:0) - (bad?1:0); */

	printf("<");
	for(i=0;i<bad;i++) printf("-");
	/* for(int i=0;i<err;i++) cout<<":"; */
	for(i=0;i<good;i++) printf("=");
	if (perc != 100)
	{
		printf (">");
		blank--;
	}
	for(i=0;i<blank;i++) printf(" ");
	printf(">\r");

	/* end progress bar */
	fflush (stdout);
}


void mono_ShowBanner (utest_world* world)
{
	printf ("%s:\n", CUNIT_BANNER);
	printf ("\t" CUNIT_URL "\n");
	printf ("Search Directory [%s]\n", world->rootDir);
	if (world->filter)
	{
		printf ("Search Filter [%s]\n", world->filter);
	}
}


#define MAX_SHOW_PATH_LEN 100
#define MAX_SHOW_NAME_LEN 100
void mono_ShowTestMessage (utest_suite_stub* stub, const char* msg)
{
	char path[MAX_SHOW_PATH_LEN];
	char name[MAX_SHOW_NAME_LEN];

	sprintf (path, "[%-30.30s]", stub->libpath);
	sprintf (name, "%s", stub->suite->name);
	printf ("%s %s %-20.20s %2d test%s\n", 
			msg,
			path,
			name,
			stub->testCount,
			stub->testCount == 1 ? "" : "s");
}


void mono_ShowAddTest (utest_suite_stub* stub)
{
	mono_ShowTestMessage (stub, "Adding suite  ");
}



void mono_ShowSkipTest (utest_suite_stub* stub)
{
	mono_ShowTestMessage (stub, "Skipping suite");
}


static ShowFunctions monoShowFunctions =
{
	&mono_ShowInit,
	&mono_ShowFini,
	&mono_ShowError_va,
	&mono_ShowMsg_va,
	&mono_ShowUsage,
	&mono_ShowBanner,
	&mono_ShowAddTest,
	&mono_ShowSkipTest,
	&mono_ShowInfo,

	NULL,
	NULL,
	&mono_ShowStart,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&mono_ShowProgress,
	&mono_ShowProgress,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

ShowFunctions* getMonoShowFunctions()
{
	return &monoShowFunctions;
}
