/* CUnitTester http://www.gethos.net/opensource/cunit
 * show_xml.c -- handles all normal xml output
 *
 * Copyright 2003 by Stewart Gebbie
 * For license terms, see the file COPYING in the top directory.
 */
/* $Id: show_xml.c,v 1.6 2005/01/05 20:53:56 stewart Exp $ */
#include "config.h"
#include "unittester.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "show_xml.h"

#define XML_NONE 0

void xml_ShowInit ()
{
	printf("<?xml version=\"1.0\"?>\n");
	printf("<Cunit>\n");
}

void xml_ShowFini ()
{
	printf("</Cunit>\n");
}

void xml_ShowError_va (const char* fmt, va_list ap)
{
	printf ("<Error>");
	vprintf (fmt, ap);
	printf ("<Error>\n");
}

void xml_ShowMsg_va (const char* fmt, va_list ap)
{
	printf ("<Message>");
	vprintf (fmt, ap);
	printf ("</Message>\n");
}

void xml_ShowUsage (int argc, char* argv[], const char* errmsg)
{
	printf ("Usage: %s [-h] [-r search directory] [-f filter regex]\n", argv[0]);
	if (errmsg) printf ("Error: %s\n", errmsg);
	printf ("\n"
			"%s\n"
			"\tURL: %s\n"
			"\n"
			"A framework for running unit tests written in C\n",
			CUNIT_BANNER,
			CUNIT_URL
			);
	printf (
			"\n"
			"\tStarting in the specified search directory, CUnitTester\n"
			"\twill recurse through the sub directories and search for\n"
			"\tfiles with names matching libtest_*.so.\n"
			"\tThe test suites defined within these libraries and matching\n"
			"\tthe filter expression are then loaded and run.\n"
			"\tThe final statistics are displayed along with any\n"
			"\taccumulated error messages.\n"
			);
}


static void xml_ShowTestInfo (utest_info* info)
{
	if (info->status == false)
	{
		printf("\t\t<Error>\n");
		printf("\t\t\t<SuiteName>%s</SuiteName>\n", info->suiteName);
		printf("\t\t\t<TestName>%s</TestName>\n", info->testName);
		printf("\t\t\t<FileName>%s</FileName>\n", info->file);
		printf("\t\t\t<LineNumber>%d</LineNumber>\n", info->line);
		printf("\t\t\t<Reason>%s</Reason>\n", info->msg);
		printf("\t\t</Error>\n");
	}
}


void xml_ShowInfo (utest_world* world)
{
	double percentGood;
	double percentBad;
	double runTime;

	utest_info* current;

	if (world->badTestCount > 0)
		printf ("\t<Errors>\n");
	for (current = world->infoHead; current != NULL; current = current->next)
		xml_ShowTestInfo (current);
	if (world->badTestCount > 0)
		printf ("\t</Errors>\n");


	percentGood = 100.0*((double)world->goodTestCount/(double)world->testCount);
	percentBad = 100.0*((double)world->badTestCount/(double)world->testCount);

	printf ("\t<Summary>\n");
	printf ("\t\t<TotalTests>\n"
			"\t\t\t<Count>%d</Count>\n"
			"\t\t\t<Percent>100</Percent>\n"
			"\t\t</TotalTests>\n", world->testCount);
	printf ("\t\t<GoodTests>\n"
			"\t\t\t<Count>%d</Count>\n"
			"\t\t\t<Percent>%3.2f</Percent>\n"
			"\t\t</GoodTests>\n",
			world->goodTestCount, percentGood);
	printf ("\t\t<BadTests>\n"
			"\t\t\t<Count>%d</Count>\n"
			"\t\t\t<Percent>%3.2f</Percent>\n"
			"\t\t</BadTests>\n",
			world->badTestCount, percentBad);

	/* calculate times */
	runTime = (world->secEnd + ((double)world->usecEnd/(double)1000000))
		- (world->secStart + ((double)world->usecStart/(double)1000000));
	printf ("\t\t<RunTime>\n\t\t\t<Seconds>%3.3f</Seconds>\n\t\t</RunTime>\n", runTime);
	printf ("\t</Summary>\n");

	printf ("\t<BottomLine>\n");
	printf ("\t\t<Good>%d</Good>\n", world->goodTestCount);
	printf ("\t\t<Total>%d</Total>\n", world->testCount);
	printf ("\t\t<Percent>%3.2f</Percent>\n", percentGood);
	printf ("\t</BottomLine>\n");
}

void xml_ShowBanner (utest_world* world)
{
	time_t now = 0;
	char now_ctime[30];
	char* newline = NULL;
	printf ("\t<Banner>%s</Banner>\n", CUNIT_BANNER);
	printf ("\t<Url>%s</Url>\n", CUNIT_URL);

	now = time(NULL);
	ctime_r (&now, now_ctime);
	newline = strstr (now_ctime, "\n");
	if (newline) *newline = '\0';
	printf ("\t<Date>\n"
			"\t\t<Epoch>%d</Epoch>\n"
			"\t\t<Timestamp>%s</Timestamp>\n"
			"\t</Date>\n", (int)now, now_ctime);
}

void xml_ShowPreCompile (utest_world* world)
{
	printf ("\t<Compile>\n");
	printf ("\t\t<SearchDirectory>%s</SearchDirectory>\n", world->rootDir);
	if (world->filter)
	{
		printf ("\t\t<SearchFilter>%s</SearchFilter>\n", world->filter);
	}
}

void xml_ShowPostCompile (utest_world* world)
{
	printf ("\t</Compile>\n");
}

void xml_ShowPreRun (utest_world* world)
{
	printf ("\t<Run>\n"
			"\t\t<TotalTests>%d</TotalTests>\n"
			"\t\t<TotalSuites>%d</TotalSuites>\n",
			world->testCount, 
			world->suiteCount
			);
}

void xml_ShowPostRun (utest_world* world)
{
	printf ("\t</Run>\n");
}

#define MAX_SHOW_PATH_LEN 100
#define MAX_SHOW_NAME_LEN 100
void xml_ShowTestMessage (utest_suite_stub* stub, const char* msg)
{
	char path[MAX_SHOW_PATH_LEN];
	char name[MAX_SHOW_NAME_LEN];

	snprintf (path, sizeof(path), "%s", stub->libpath);
	snprintf (name, sizeof(path), "%s", stub->suite->name);

	printf("\t\t<%s>\n"
		   "\t\t\t<Path>%s</Path>\n"
		   "\t\t\t<Name>%s</Name>\n"
		   "\t\t\t<Count>%d</Count>\n"
		   "\t\t</%s>\n",
			msg,
			path,
			name,
			stub->testCount,
			msg);
}


void xml_ShowAddTest (utest_suite_stub* stub)
{
	xml_ShowTestMessage (stub, "AddingSuite");
}



void xml_ShowSkipTest (utest_suite_stub* stub)
{
	xml_ShowTestMessage (stub, "SkippingSuite");
}


void xml_ShowPreSuite (utest_suite* suite)
{
	printf ("\t\t<Suite>\n");
	printf ("\t\t\t<Name>%s</Name>\n", suite->name);
}

void xml_ShowPostSuite (utest_suite* suite)
{
	printf ("\t\t</Suite>\n");
}

static void xml_ShowPostStatus (utest_world* world, utest_info* info)
{
	if (info->status == false)
	{
		printf ("\t\t\t\t\t<Status>Bad</Status>\n");
		printf ("\t\t\t\t\t<Message>%s</Message>\n", info->msg);
		printf ("\t\t\t\t\t<FileName>%s</FileName>\n", info->file);
		printf ("\t\t\t\t\t<LineNumber>%d</LineNumber>\n", info->line);
	}
	else
	{
		printf ("\t\t\t\t\t<Status>Good</Status>\n");
	}
}

void xml_ShowPreInit (utest_world* world)
{
	printf ("\t\t\t<Init>\n");
}

void xml_ShowPostInit (utest_world* world, utest_info* info)
{
	printf ("\t\t\t</Init>\n");
}

void xml_ShowPreFixture (utest_world* world, utest_test* test)
{
	printf ("\t\t\t<Fixture>\n");
	printf ("\t\t\t\t<Name>%s</Name>\n", test->name);
}

void xml_ShowPostFixture (utest_world* world, utest_test* test)
{
	printf ("\t\t\t</Fixture>\n");
}

void xml_ShowPreSetup (utest_world* world)
{
	printf ("\t\t\t\t<Setup>\n");
}

void xml_ShowPostSetup (utest_world* world, utest_info* info)
{
	xml_ShowPostStatus (world, info);
	printf ("\t\t\t\t</Setup>\n");
}

void xml_ShowPreTest (utest_world* world)
{
	printf ("\t\t\t\t<Test>\n");
}

void xml_ShowPostTest (utest_world* world, utest_info* info)
{
	xml_ShowPostStatus (world, info);
	printf ("\t\t\t\t</Test>\n");
}

void xml_ShowPreTeardown (utest_world* world)
{
	printf ("\t\t\t\t<Teardown>\n");
}

void xml_ShowPostTeardown (utest_world* world, utest_info* info)
{
	xml_ShowPostStatus (world, info);
	printf ("\t\t\t\t</Teardown>\n");
}

void xml_ShowPreFini (utest_world* world)
{
	printf ("\t\t\t<Fini>\n");
}

void xml_ShowPostFini (utest_world* world, utest_info* info)
{
	xml_ShowPostStatus (world, info);
	printf ("\t\t\t</Fini>\n");
}



static ShowFunctions xmlShowFunctions =
{
	&xml_ShowInit,
	&xml_ShowFini,
	&xml_ShowError_va,
	&xml_ShowMsg_va,
	&xml_ShowUsage,
	&xml_ShowBanner,
	&xml_ShowAddTest,
	&xml_ShowSkipTest,
	&xml_ShowInfo,

	&xml_ShowPreCompile,
	&xml_ShowPostCompile,
	&xml_ShowPreRun,
	&xml_ShowPostRun,
	&xml_ShowPreSuite,
	&xml_ShowPostSuite,
	&xml_ShowPreInit,
	&xml_ShowPostInit,
	&xml_ShowPreFixture,
	&xml_ShowPostFixture,
	&xml_ShowPreSetup,
	&xml_ShowPostSetup,
	&xml_ShowPreTest,
	&xml_ShowPostTest,
	&xml_ShowPreTeardown,
	&xml_ShowPostTeardown,
	&xml_ShowPreFini,
	&xml_ShowPostFini
};

ShowFunctions* getXmlShowFunctions()
{
	return &xmlShowFunctions;
}
